<?php //csvfile.inc.php (table-xs v1.00)

class csvfile 
/*
   Die Datenquelle ist eine CSV-Textdatei (comma separated values)
  
   Alle Werte stehen in Gnsefsschen (") und werden jeweils
   durch Kommata (,) voneinander getrennt. ("...","...","...")
  
   Die erste Zeile enthlt die Attribute (Tabellenberschriften)     
   
*/
{

	// ffentliche Klassenvariablen, auf die von aussen zugegriffen werden kann
	var $name	= "";
	var $handle	= "";
	var $captions   = array();


//-------------------------------- Hilfsfunktionen -------------------------------------

	function clean_string( $str )
	// Aufgabe:  Entfernt alle berflssigen Trenn- und Sonderzeichen 
	//           aus dem String
	//
	// Eingabe:  $str   : Ein aus der CSV-Datei eingelesener Teilstring
	//
	// Rckgabe: String ohne " (Quotes) an Anfang und Ende
	//           (ausserdem wurden alle slash-kodierten Zeichen dekodiert)
	{

		$str = trim( $str );
		$strlen = strlen( $str );
		
		// falls der String mit Quotes beginnt, die Quotes ignorieren
		substr($str,0,1) == '"' ? $start = 1 : $start = 0;
		
		// falls der String mit Quotes endet, auch diese Quotes ignorieren
		if (substr( $str, $strlen-1, 1 ) == '"') { $strlen--; }
		
		// den "eigentlichen" String ausschneiden !!!
		$str = substr( $str, $start, $strlen );
		
		$str = stripslashes( $str );	// slash-kodierte Sonderzeichen dekodieren
		
		return $str;
	}

	function get_captions()
	// Aufgabe:  Ermittelt die Tabellenberschriften aus den
	//	     Eintrgen der 1. Zeile der CSV-Datei
	//
	// Eingabe:  keine
	//
	// Rckgabe: Rckgabe der Tabellenberschriften in $captions
	{
		$line = fgets( $this->handle, 4096 );
		$tokens = explode( '","' , $line );
		
		$keys = array_keys( $tokens );
		foreach ($keys as $key)
		{
			$this->captions[$key] = $this->clean_string( $tokens[$key] );
		}		
		
	}



//--------------------------------- Implementation des Interface ----------------------


	function open( $mode )
	// ffnet die CSV-Datei (im angegebenen Dateimodus)
	
	{
		if ( ! $this->handle = @fopen( $this->name, $mode ) ) 
		{ 
			echo "Fehler: Konnte  Datei '$this->name' nicht ffnen!";
			return false;
		} else 
		   {
		     	return true;
		   }
	}

	
	function close()
	// Schliet die CSV-Datei

	{
		return fclose( $this->handle );
	}

	
	function init()
	// Initialisiert die Datenquelle

	{
		if ( $this->open( "a+" ) && $this->close() )
		{
//			echo "Datei <b>$this->name</b> steht zur Verfgung...<br>";
			return true;
		}
		return false;
	}


	function entries()
	// Aufgabe:  Ermittelt die Anzahl der Eintrge (Datenstze)
	//
	// Eingabe:  keine
	//
	// Rckgabe: Anzahl der Eintrge (Datenstze)

	{
		$num_entries = 0;
		$this->open( "r" );

		// Tabellenberschriften aus der ersten Zeile einlesen
		$this->get_captions();

		// Datei zeilenweise durchlaufen, und dabei mitzhlen!
		while ( $line = fgets( $this->handle, 4096 ) )
		{ 
			$num_entries++; 
		}
		$this->close();	
		return $num_entries;		
	}


	function get_entry( $pos, &$rows )
	// Aufgabe:  Holt einen bestimmten Eintrag aus der Datei
	//
	// Eingabe:  $line	= Nummer der zu holenden Tabellenzeile
	//
	// Rckgabe: $rows 	= Array mit den gelesenen Eintrgen
	{

		$this->open( "r" );

		// Tabellenberschriften aus der ersten Zeile einlesen
		$this->get_captions();
	
		// Alle Eintrge bis zur Startposition berlesen
		$i=0;
		while ( ($i < $pos) && ($line = fgets( $this->handle, 4096 )))
		{
	  		$i++;	
		}

		$line = fgets( $this->handle, 4096 );
		$this->close();
		
		$tokens = explode( '","' , $line );

		$keys = array_keys( $tokens );
		reset( $this->captions );

		foreach ($keys as $key)
		{
			$cap = current( $this->captions );
			$rows[$cap] = $this->clean_string( $tokens[$key] );
			next( $this->captions );
		}

	}


	function get_entrylist( $start, $stop, &$entrylist )
	// Aufgabe:  Liefert einen bestimmten Ausschnitt der Tabelle
	//
	// Eingabe:  $start	= Erster zu holender Eintrag
	//           $end	= Letzter zu holender Eintrag
	//
	// Rckgabe: $entrylist = Array mit den gelesenen Eintrgen
	{

		$this->open( "r" );

		// Tabellenberschriften aus der ersten Zeile einlesen
		$this->get_captions();
	
		// Alle Eintrge bis zur Startposition berlesen
		$i=0;
		while ( ($i < $start) && ($line = fgets( $this->handle, 4096 )))
		{
	  		$i++;	
		}

		while ( ($i <= $stop) && ($line = fgets( $this->handle, 4096 )) )
		{

			$tokens = explode( '","' , $line );

			$keys = array_keys( $tokens );
			reset( $this->captions );

			foreach ($keys as $key)
			{
				$cap = current( $this->captions );
				$entrylist[$i][$cap] = $this->clean_string( $tokens[$key] );
				next( $this->captions );
			}

			$i++;
		}

		$this->close();		

	}

	
	function append( $data )
	// Aufgabe:  Hngt die Daten an das Ende der Tabelle
	//
	// Eingabe:  $data	= Array mit den zu speichernden Daten
	//
	{

		clearstatcache();		// unbedingt ntigt, da sonst filesize konstant bleibt!!!
		if ( filesize( $this->name ) == 0 )
		{
			$line = '';
			$this->open( "a+" );

			$keys = array_keys( $data );
			foreach ($keys as $key)
			{
			
				if (!empty( $line )) 
				{ 
					$line .= ','; 
				}
				$line .= '"' . $key. '"';
			}
		
			fputs( $this->handle, $line."\n" /*.chr(13).chr(10)*/ );
			$this->close();
		  
		} 
		
		$this->open( "r" );
		$this->get_captions();
		$this->close();

		$line = '';
		$this->open( "a+" );

		$values = array_values( $this->captions );
		foreach ($values as $val)
		{
			if (!empty( $line )) 
			{ 
				$line .= ','; 
			}
			$line .= '"' . $data[$val]. '"';
		}
		
		fputs( $this->handle, $line."\n" /*chr(13).chr(10)*/ );
		$this->close();
		
	}

	
	function find_entry( $data )
	// Aufgabe:  Sucht nach dem ersten Auftreten eines Datums
	//
	// Eingabe:  $data	= Gesuchtes Datum
	//
	{
		$ok = false; $entry = 0;
		$this->open( "r" );
		$this->get_captions();
		while ( $line = fgets( $this->handle, 4096 ) AND (!$ok) )
		{
			if ( strstr( $line, $data ) )
			{
				$ok = true; 
			} else { $entry++; }
			
		}

		$this->close();
		return $entry;
		
	}
	
	function change( $pos, $data )
	// Aufgabe:  ndert eine bestimmte Zeile in der Tabelle
	//
	// Eingabe:	$pos	= Position des Elements
	//		$data	= Array mit den zu speichernden Daten
	//
	{

		$num_of_entries = $this->entries();

		if ( filesize( $this->name ) == 0 )
		{
			$this->append( $data );
		} 
		
		$this->open( "r" );
		$this->get_captions();
		$this->close();
		
		// Komplette Datei zeilenweise einlesen
		$entrylist = file( $this->name );
		
		$newline = '';

		$values = array_values( $this->captions );
		foreach ($values as $val)
		{
			if (!empty( $newline )) 
			{ 
				$newline .= ','; 
			}
			$newline .= '"' . $data[$val]. '"';
		}
		$newline .= "\n";
		$entrylist[$pos+1]=$newline;
		
		$this->open( "w" );
		fwrite( $this->handle, ereg_replace( "\r", "", implode( "", $entrylist ) ) );


		$this->close();		
	
	}

	function delete( $pos )
	// Aufgabe:  Lscht eine bestimmte Zeile in der Tabelle
	//
	// Eingabe:	$pos	= Position des Elements
	//
	{
		$num_of_entries = $this->entries();

		if ( filesize( $this->name ) != 0 )
		{
		
			// Komplette Datei zeilenweise einlesen
			$entrylist = file( $this->name );
		
			unset( $entrylist[$pos+1] );
		
			$this->open( "w" );
			fwrite( $this->handle, ereg_replace( "\r", "", implode( "", $entrylist ) ) );

			$this->close();		
	
		}
	}



}

?>